@@ -13,8 +13,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 if ( $argc <= 1 || $argc > 4 )
 {
@@ -50,6 +48,7 @@ foreach( $functions as $name => $f )
 		$maxLen = strlen( $name );
 	}
 }
+$maxLen = max( $maxLen, 8 );
 
 echo "Showing the {$elements} most costly calls sorted by '{$sortKey}'.\n\n";
 
@@ -127,11 +126,12 @@ class drXdebugTraceFileParser
 		$size = fstat( $this->handle );
 		$size = $size['size'];
 		$read = 0;
-		
+
 		while ( !feof( $this->handle ) )
 		{
 			$buffer = fgets( $this->handle, 4096 );
 			$read += strlen( $buffer );
+			$buffer = rtrim( $buffer, PHP_EOL );
 			$this->parseLine( $buffer );
 			$c++;
 
@@ -184,6 +184,10 @@ class drXdebugTraceFileParser
 				$dTime   = $time   - $prevTime;
 				$dMemory = $memory - $prevMem;
 
+				if ( ! array_key_exists( $depth - 1, $this->stack ) )
+				{
+					$this->stack[$depth - 1] = array( '', 0, 0, 0, 0 );
+				}
 				$this->stack[$depth - 1][3] += $dTime;
 				$this->stack[$depth - 1][4] += $dMemory;
 
@@ -229,7 +233,7 @@ class drXdebugTraceFileParser
 
 		if ( $sortKey !== null )
 		{
-			uasort( $result, 
+			uasort( $result,
 				function( $a, $b ) use ( $sortKey )
 				{
 					return ( $a[$sortKey] > $b[$sortKey] ) ? -1 : ( $a[$sortKey] < $b[$sortKey] ? 1 : 0 );
